/******************************************************************************
 *                               G R L _ V 1 . h
 *-----------------------------------------------------------------------------
 *   Inria Grenoble - Rhone-Alpes
 *   655, avenue de l'Europe
 *   38330 Montbonnot Saint Martin
 *   FRANCE
 *-----------------------------------------------------------------------------
 *   Module             :       GRL_V1.h
 *   Auteur             :       Eric Leo
 *   Version            :       1.4
 *   Date               :       15/11/06 12:42:54
 *****************************************************************************/

/*****************************************************************************
 *
 * This library is used to interface C code with GRL tools.
 *
 *****************************************************************************/

/*---------------------------------------------------------------------------*/
/*                           Type definitions                                */
/*---------------------------------------------------------------------------*/

/* types matching GRL types */
typedef ADT_RANGE__NAT8  GRL_NAT;       /* refer to ADT unsigned char */
typedef ADT_RANGE__NAT16 GRL_NAT16;     /* refer to ADT unsigned short */
typedef ADT_NAT          GRL_NAT32;     /* refer to ADT unsigned int */
typedef ADT_RANGE__INT8  GRL_INT;       /* refer to ADT unsigned char */
typedef ADT_RANGE__INT16 GRL_INT16;     /* refer to ADT unsigned short */
typedef ADT_INT          GRL_INT32;     /* refer to ADT signed int */
typedef ADT_STRING       GRL_STRING;    /* refer to ADT */
typedef ADT_CHAR         GRL_CHAR;      /* refer to ADT char */
typedef ADT_BOOL         GRL_BOOL;      /* refer to ADT unsigned char */

/* specific types */
typedef ADT_WIDE_INT     GRL_WIDE_INT;  /* refer to ADT */

/*---------------------------------------------------------------------------*/
/*                           Interval definitions                            */
/*---------------------------------------------------------------------------*/

/* GRL_NAT */
#define GRL_HIGH_NAT UCHAR_MAX
#define GRL_LOW_NAT ADT_LOW_NAT

/* GRL_NAT16 */
#define GRL_HIGH_NAT16 USHRT_MAX
#define GRL_LOW_NAT16 GRL_LOW_NAT

/* GRL_NAT32 */
#define GRL_HIGH_NAT32 ADT_HIGH_NAT
#define GRL_LOW_NAT32 GRL_LOW_NAT

/* GRL_INT */
#define GRL_HIGH_INT GRL_HIGH_NAT
#define GRL_LOW_INT GRL_LOW_NAT

/* GRL_INT16 */
#define GRL_HIGH_INT16 GRL_HIGH_NAT16
#define GRL_LOW_INT16 GRL_LOW_NAT16

/* GRL_INT32 */
#define GRL_HIGH_INT32 ADT_HIGH_INT
#define GRL_LOW_INT32 ADT_LOW_INT

/* GRL_CHAR */
#define GRL_HIGH_CHAR SHRT_MAX
#define GRL_LOW_CHAR SHRT_MIN

/* GRL_BOOL */
#define GRL_HIGH_BOOL ADT_TRUE()
#define GRL_LOW_BOOL ADT_FALSE()

/* unsigned char */
#define GRL_HIGH_UC UCHAR_MAX
#define GRL_LOW_UC GRL_LOW_NAT

/* unsigned short */
#define GRL_HIGH_US USHRT_MAX
#define GRL_LOW_US GRL_LOW_NAT

/* unsigned int */
#define GRL_HIGH_UI UINT_MAX
#define GRL_LOW_UI GRL_LOW_NAT

/* signed char */
#define GRL_HIGH_SC SCHAR_MAX
#define GRL_LOW_SC SCHAR_MIN

/* signed short */
#define GRL_HIGH_SS SHRT_MAX
#define GRL_LOW_SS SHRT_MIN

/* signed int */
#define GRL_HIGH_SI INT_MAX
#define GRL_LOW_SI INT_MIN

/*---------------------------------------------------------------------------*/
/*                           Interface functions                             */
/*---------------------------------------------------------------------------*/

#define GRL_NAT_TO_C(X)    (X + (GRL_LOW_UC - ADT_RANGE_ENUM_FIRST__NAT8()))
#define GRL_NAT16_TO_C(X)  (X + (GRL_LOW_US - ADT_RANGE_ENUM_FIRST__NAT16()))
#define GRL_NAT32_TO_C(X)  (X)
#define GRL_INT_TO_C(X)    (X + (GRL_LOW_SC - ADT_RANGE_ENUM_FIRST__INT8()))
#define GRL_INT16_TO_C(X)  (X + (GRL_LOW_SS - ADT_RANGE_ENUM_FIRST__INT16()))
#define GRL_INT32_TO_C(X)  (X)

#define GRL_C_TO_NAT(X)    (X - (GRL_LOW_UC - ADT_RANGE_ENUM_FIRST__NAT8()))
#define GRL_C_TO_NAT16(X)  (X - (GRL_LOW_US - ADT_RANGE_ENUM_FIRST__NAT16()))
#define GRL_C_TO_NAT32(X)  (X)
#define GRL_C_TO_INT(X)    (X - (GRL_LOW_SC - ADT_RANGE_ENUM_FIRST__INT8()))
#define GRL_C_TO_INT16(X)  (X - (GRL_LOW_SS - ADT_RANGE_ENUM_FIRST__INT16()))
#define GRL_C_TO_INT32(X)  (X)

#define GRL_C_TO_STRING(X) (X)
#define GRL_STRING_TO_C(X) (X)

#define GRL_C_TO_CHAR(X)   (X)
#define GRL_CHAR_TO_C(X)   (X)

#define GRL_C_TO_BOOL(X)   (X)
#define GRL_BOOL_TO_C(X)   (X)


/*---------------------------------------------------------------------------*/
/*                           Error functions                                 */
/*---------------------------------------------------------------------------*/

void GRL_ERROR_OUT_OF_BOUNDS (signed long long X,
                              signed long LOW,
                              signed long HIGH)
{
   fprintf (stdout,
            "#402 error in file ``%s'' at line %d:\n     integer value %lld \
out of the range [%ld .. %ld]\n",
            __FILE__,
            __LINE__,
            X,
            LOW,
            HIGH);
   fflush (stdout);
   raise (15);
}

/*---------------------------------------------------------------------------*/
/*                           Check functions                                 */
/*---------------------------------------------------------------------------*/

#define GRL_INVALID_NAT(X)   (GRL_HIGH_NAT < (X))
#define GRL_INVALID_NAT16(X) (GRL_HIGH_NAT16 < (X))
#define GRL_INVALID_NAT32(X) (GRL_HIGH_NAT32 < (X))
#define GRL_INVALID_INT(X)   (((X) < GRL_LOW_INT) || (GRL_HIGH_INT < (X)))
#define GRL_INVALID_INT16(X) (((X) < GRL_LOW_INT16) || (GRL_HIGH_INT16 < (X)))
#define GRL_INVALID_INT32(X) (((X) < GRL_LOW_INT32) || (GRL_HIGH_INT32 < (X)))
#define GRL_INVALID_CHAR(X)  (((X) < GRL_LOW_CHAR) || (GRL_HIGH_CHAR < (X)))
#define GRL_INVALID_BOOL(X)  (((X) < GRL_LOW_BOOL) || (GRL_HIGH_BOOL < (X)))
#define GRL_INVALID_UC(X)    (GRL_HIGH_UC < (X))
#define GRL_INVALID_US(X)    (GRL_HIGH_US < (X))
#define GRL_INVALID_UI(X)    (GRL_HIGH_UI < (X))
#define GRL_INVALID_SC(X)    (((X) < GRL_LOW_SC) || (GRL_HIGH_SC < (X)))
#define GRL_INVALID_SS(X)    (((X) < GRL_LOW_SS) || (GRL_HIGH_SS < (X)))
#define GRL_INVALID_SI(X)    (((X) < GRL_LOW_SI) || (GRL_HIGH_SI < (X)))

GRL_WIDE_INT GRL_CHECK_NAT (GRL_WIDE_INT X)
{
   if (GRL_INVALID_NAT (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_NAT, GRL_HIGH_NAT);
   return X;
}

GRL_WIDE_INT GRL_CHECK_NAT16 (GRL_WIDE_INT X)
{
   if (GRL_INVALID_NAT16 (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_NAT16, GRL_HIGH_NAT16);
   return X;
}

GRL_WIDE_INT GRL_CHECK_NAT32 (GRL_WIDE_INT X)
{
   if (GRL_INVALID_NAT32 (X))
     GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_NAT32, GRL_HIGH_NAT32);
   return X;
}

GRL_WIDE_INT GRL_CHECK_INT (GRL_WIDE_INT X)
{
   if (GRL_INVALID_INT (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_INT, GRL_HIGH_INT);
   return X;
}

GRL_WIDE_INT GRL_CHECK_INT16 (GRL_WIDE_INT X)
{
   if (GRL_INVALID_INT16 (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_INT16, GRL_HIGH_INT16);
   return X;
}

GRL_WIDE_INT GRL_CHECK_INT32 (GRL_WIDE_INT X)
{
   if (GRL_INVALID_INT32 (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_INT32, GRL_HIGH_INT32);
   return X;
}

GRL_WIDE_INT GRL_CHECK_CHAR (GRL_WIDE_INT X)
{
   if (GRL_INVALID_CHAR (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_CHAR, GRL_HIGH_CHAR);
   return X;
}

GRL_WIDE_INT GRL_CHECK_BOOL (GRL_WIDE_INT X)
{
   if (GRL_INVALID_BOOL (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_BOOL, GRL_HIGH_BOOL);
   return X;
}

GRL_WIDE_INT GRL_CHECK_UI (GRL_WIDE_INT X)
{
   if (GRL_INVALID_UI (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_UI, GRL_HIGH_UI);
   return X;
}

GRL_WIDE_INT GRL_CHECK_US (GRL_WIDE_INT X)
{
   if (GRL_INVALID_US (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_US, GRL_HIGH_US);
   return X;
}

GRL_WIDE_INT GRL_CHECK_UC (GRL_WIDE_INT X)
{
   if (GRL_INVALID_UC (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_UC, GRL_HIGH_UC);
   return X;
}

GRL_WIDE_INT GRL_CHECK_SI (GRL_WIDE_INT X)
{
   if (GRL_INVALID_SI (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_SI, GRL_HIGH_SI);
   return X;
}

GRL_WIDE_INT GRL_CHECK_SS (GRL_WIDE_INT X)
{
   if (GRL_INVALID_SS (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_SS, GRL_HIGH_SS);
   return X;
}

GRL_WIDE_INT GRL_CHECK_SC (GRL_WIDE_INT X)
{
   if (GRL_INVALID_SC (X))
      GRL_ERROR_OUT_OF_BOUNDS (X, GRL_LOW_SC,  GRL_HIGH_SC);
   return X;
}

/*---------------------------------------------------------------------------*/
/*                           Conversion functions from GRL types             */
/*---------------------------------------------------------------------------*/

#define GRL_NAT_TO_UNSIGNED_INT(X)     ((unsigned int) GRL_CHECK_UI (GRL_NAT_TO_C (GRL_CHECK_NAT (X))))
#define GRL_NAT16_TO_UNSIGNED_INT(X)   ((unsigned int) GRL_CHECK_UI (GRL_NAT16_TO_C (GRL_CHECK_NAT16 (X))))
#define GRL_NAT32_TO_UNSIGNED_INT(X)   ((unsigned int) GRL_CHECK_UI (GRL_NAT32_TO_C (GRL_CHECK_NAT32 (X))))
#define GRL_NAT_TO_UNSIGNED_SHORT(X)   ((unsigned short) GRL_CHECK_US (GRL_NAT_TO_C (GRL_CHECK_NAT (X))))
#define GRL_NAT16_TO_UNSIGNED_SHORT(X) ((unsigned short) GRL_CHECK_US (GRL_NAT16_TO_C (GRL_CHECK_NAT16 (X))))
#define GRL_NAT32_TO_UNSIGNED_SHORT(X) ((unsigned short) GRL_CHECK_US (GRL_NAT32_TO_C (GRL_CHECK_NAT32 (X))))
#define GRL_NAT_TO_UNSIGNED_CHAR(X)    ((unsigned char) GRL_CHECK_UC (GRL_NAT_TO_C (GRL_CHECK_NAT (X))))
#define GRL_NAT16_TO_UNSIGNED_CHAR(X)  ((unsigned char) GRL_CHECK_UC (GRL_NAT16_TO_C (GRL_CHECK_NAT16 (X))))
#define GRL_NAT32_TO_UNSIGNED_CHAR(X)  ((unsigned char) GRL_CHECK_UC (GRL_NAT32_TO_C (GRL_CHECK_NAT32 (X))))
#define GRL_INT_TO_SIGNED_INT(X)       ((signed int) GRL_CHECK_SI (GRL_INT_TO_C (GRL_CHECK_INT (X))))
#define GRL_INT16_TO_SIGNED_INT(X)     ((signed int) GRL_CHECK_SI (GRL_INT16_TO_C (GRL_CHECK_INT16 (X))))
#define GRL_INT32_TO_SIGNED_INT(X)     ((signed int) GRL_CHECK_SI (GRL_INT32_TO_C (GRL_CHECK_INT32 (X))))
#define GRL_INT_TO_SIGNED_SHORT(X)     ((signed short) GRL_CHECK_SS (GRL_INT_TO_C (GRL_CHECK_INT (X))))
#define GRL_INT16_TO_SIGNED_SHORT(X)   ((signed short) GRL_CHECK_SS (GRL_INT16_TO_C (GRL_CHECK_INT16 (X))))
#define GRL_INT32_TO_SIGNED_SHORT(X)   ((signed short) GRL_CHECK_SS (GRL_INT32_TO_C (GRL_CHECK_INT32 (X))))
#define GRL_INT_TO_SIGNED_CHAR(X)      ((signed char) GRL_CHECK_SC (GRL_INT_TO_C (GRL_CHECK_INT (X))))
#define GRL_INT16_TO_SIGNED_CHAR(X)    ((signed char) GRL_CHECK_SC (GRL_INT16_TO_C (GRL_CHECK_INT16 (X))))
#define GRL_INT32_TO_SIGNED_CHAR(X)    ((signed char) GRL_CHECK_SC (GRL_INT32_TO_C (GRL_CHECK_INT32 (X))))

#define GRL_STRING_TO_CHAR_PTR(X)      ((char*) GRL_STRING_TO_C (X))

#define GRL_CHAR_TO_SIGNED_CHAR(X)     ((signed char) GRL_CHECK_SC (GRL_CHAR_TO_C (GRL_CHECK_CHAR (X))))

#define GRL_BOOL_TO_UNSIGNED_CHAR(X)   ((unsigned char) GRL_CHECK_UC (GRL_BOOL_TO_C (GRL_CHECK_BOOL (X))))
#define GRL_BOOL_TO_UNSIGNED_SHORT(X)  ((unsigned short) GRL_CHECK_US (GRL_BOOL_TO_C (GRL_CHECK_BOOL (X))))
#define GRL_BOOL_TO_UNSIGNED_INT(X)    ((unsigned int) GRL_CHECK_UI (GRL_BOOL_TO_C (GRL_CHECK_BOOL (X))))

/*---------------------------------------------------------------------------*/
/*                           Conversion functions to GRL types               */
/*---------------------------------------------------------------------------*/

#define GRL_UNSIGNED_INT_TO_NAT(X)     ((GRL_NAT) GRL_CHECK_NAT (GRL_C_TO_NAT (GRL_CHECK_UI (X))))
#define GRL_UNSIGNED_INT_TO_NAT16(X)   ((GRL_NAT16) GRL_CHECK_NAT16 (GRL_C_TO_NAT16 (GRL_CHECK_UI (X))))
#define GRL_UNSIGNED_INT_TO_NAT32(X)   ((GRL_NAT32) GRL_CHECK_NAT32 (GRL_C_TO_NAT32 (GRL_CHECK_UI (X))))
#define GRL_UNSIGNED_SHORT_TO_NAT(X)   ((GRL_NAT) GRL_CHECK_NAT (GRL_C_TO_NAT (GRL_CHECK_US (X))))
#define GRL_UNSIGNED_SHORT_TO_NAT16(X) ((GRL_NAT16) GRL_CHECK_NAT16 (GRL_C_TO_NAT16 (GRL_CHECK_US (X))))
#define GRL_UNSIGNED_SHORT_TO_NAT32(X) ((GRL_NAT32) GRL_CHECK_NAT32 (GRL_C_TO_NAT32 (GRL_CHECK_US (X))))
#define GRL_UNSIGNED_CHAR_TO_NAT(X)    ((GRL_NAT) GRL_CHECK_NAT (GRL_C_TO_NAT (GRL_CHECK_UC (X))))
#define GRL_UNSIGNED_CHAR_TO_NAT16(X)  ((GRL_NAT16) GRL_CHECK_NAT16 (GRL_C_TO_NAT16 (GRL_CHECK_UC (X))))
#define GRL_UNSIGNED_CHAR_TO_NAT32(X)  ((GRL_NAT32) GRL_CHECK_NAT32 (GRL_C_TO_NAT32 (GRL_CHECK_UC (X))))
#define GRL_SIGNED_INT_TO_INT(X)       ((GRL_INT) GRL_CHECK_INT (GRL_C_TO_INT (GRL_CHECK_SI (X))))
#define GRL_SIGNED_INT_TO_INT16(X)     ((GRL_INT16) GRL_CHECK_INT16 (GRL_C_TO_INT16 (GRL_CHECK_SI (X))))
#define GRL_SIGNED_INT_TO_INT32(X)     ((GRL_INT32) GRL_CHECK_INT32 (GRL_C_TO_INT32 (GRL_CHECK_SI (X))))
#define GRL_SIGNED_SHORT_TO_INT(X)     ((GRL_INT) GRL_CHECK_INT (GRL_C_TO_INT (GRL_CHECK_SS (X))))
#define GRL_SIGNED_SHORT_TO_INT16(X)   ((GRL_INT16) GRL_CHECK_INT16 (GRL_C_TO_INT16 (GRL_CHECK_SS (X))))
#define GRL_SIGNED_SHORT_TO_INT32(X)   ((GRL_INT32) GRL_CHECK_INT32 (GRL_C_TO_INT32 (GRL_CHECK_SS (X))))
#define GRL_SIGNED_CHAR_TO_INT(X)      ((GRL_INT) GRL_CHECK_INT (GRL_C_TO_INT (GRL_CHECK_SC (X))))
#define GRL_SIGNED_CHAR_TO_INT16(X)    ((GRL_INT16) GRL_CHECK_INT16 (GRL_C_TO_INT16 (GRL_CHECK_SC (X))))
#define GRL_SIGNED_CHAR_TO_INT32(X)    ((GRL_INT32) GRL_CHECK_INT32 (GRL_C_TO_INT32 (GRL_CHECK_SC (X))))

#define GRL_CHAR_PTR_TO_STRING(X)      ((GRL_STRING) GRL_C_TO_STRING (X))

#define GRL_SIGNED_CHAR_TO_CHAR(X)     ((GRL_CHAR) GRL_CHECK_CHAR (GRL_C_TO_CHAR (GRL_CHECK_SC (X))))

#define GRL_UNSIGNED_CHAR_TO_BOOL(X)   ((GRL_BOOL) GRL_CHECK_BOOL (GRL_C_TO_BOOL (GRL_CHECK_UC (X))))
#define GRL_UNSIGNED_SHORT_TO_BOOL(X)  ((GRL_BOOL) GRL_CHECK_BOOL (GRL_C_TO_BOOL (GRL_CHECK_US (X))))
#define GRL_UNSIGNED_INT_TO_BOOL(X)    ((GRL_BOOL) GRL_CHECK_BOOL (GRL_C_TO_BOOL (GRL_CHECK_UI (X))))
