'\" t
.\" @(#)grl.l - CONVECS team - 16/01/22
.TH GRL LOCAL "16/01/22" "(C) INRIA" "CADP MANUAL PAGES"
.fp 4 CB
.lg 0

.cs 4 23

.SH NAME
grl \- specification language for globally asynchronous locally synchronous 
system

.SH DESCRIPTION 
.P
GRL (GALS Representation Language) is a formal imperatively styled 
specification language for GALS systems.

.SH LEXICAL ELEMENTS
.P
An identifier is defined by a letter followed by a possibly empty series of 
letters, digits and underscores. GRL prohibits that an identifier starts or 
ends with an underscore.
.P
Identifiers are case sensitive, so that all occurrences of the same identifier 
must use exactly the same case, i.e., lower-case and upper-case characters have 
to be respected. For instance, if a variable has identifier \fBXyZ\fP, then all 
its occurrences must have the same identifier \fBXyZ\fP, but neither \fBxyz\fP
nor \fBXyz\fP. However, to avoid confusion, GRL forbids declaring in the same 
scope identifiers of the same nature (e.g., variables, constructors, functions, 
etc.) differring only by their case. Therefore identifiers are not considered 
as distinct if they differ only by their case.
.P
Comments can be both:
.IP -
single line '\fB--\fP <text>'
.IP -
multi-line '\fB(*\fP <text> \fB*)\fP'
.P
Keywords must be written in lowercase.
Reserved words can not be used as identifiers in GRL programs. 
They are listed below.
.TS
center;
L L L L.
\fBabs\fP	\fBalias\fP	\fBand\fP	\fBany\fP
\fBas\fP	\fBarray\fP	\fBblock\fP	\fBbool\fP
\fBby\fP	\fBcase\fP	\fBchar\fP	\fBconst\fP
\fBelse\fP	\fBelsif\fP	\fBenable\fP	\fBend\fP
\fBenvironment\fP	\fBenum\fP	\fBequ\fP	\fBfalse\fP
\fBfor\fP	\fBfrom\fP	\fBif\fP	\fBimplies\fP
\fBin\fP	\fBint\fP	\fBint8\fP	\fBint16\fP
\fBint32\fP	\fBis\fP	\fBlist\fP	\fBloop\fP
\fBmedium\fP	\fBmodule\fP	\fBnat\fP	\fBnat8\fP
\fBnat16\fP	\fBnat32\fP	\fBnot\fP	\fBnull\fP
\fBof\fP	\fBor\fP	\fBout\fP	\fBrange\fP
\fBreceive\fP	\fBrecord\fP	\fBselect\fP	\fBsend\fP
\fBstatic\fP	\fBstring\fP	\fBsystem\fP	\fBthen\fP
\fBtrue\fP	\fBto\fP	\fBtype\fP	\fBvar\fP
\fBwhen\fP	\fBwhere\fP	\fBwhile\fP	\fBxor\fP
.TE

.SH SYNTAX DESCRIPTION
.P
The syntax of GRL is described using Bachus-Naur Form (BNF) notations. 
.IP -
Non-terminal symbols are written in italic (e.g., \fItype_def\fP).
.IP -
Keywords are written using lowercase characters and between double quotes 
(e.g., "module", "static", "if", etc.).
.br
Key symbols are written between double quotes (e.g., "|", "(", "[", "*", etc.).
.IP -
Optional sequences of symbols are written between square brackets.
.IP -
Star (*) denotes repetition zero or more times of the preceding expression.
.br
Plus (+) denotes repetition one or more times of the preceding expression.
.P
The following table summarizes the generic terminal symbols and the most 
frequently used non-terminal symbols and their meaning.
.P
.cs I 23
.nf
\fB
                  +--------+------------------------------+
                  | Symbol |   Description                |
       +----------+--------+------------------------------+
       |          |   \fIP\fP    | module                       |
       |          |   \fIS\fP    | system                       |
       |          |   \fIB\fP    | block                        |
       |          |   \fIBi\fP   | block instance               |
       |          |   \fIN\fP    | environment                  |
       |          |   \fINi\fP   | environment instance         |
       | Terminal |   \fIM\fP    | medium                       |
       |          |   \fIMi\fP   | medium instance              |
       |          |   \fIF\fP    | function                     |
       |          |   \fIX, Y\fP | variables                    |
       |          |   \fIT\fP    | user defined type identifier |
       |          |   \fItype\fP | type identifier              |
       |          |   \fIC\fP    | type constructor             |
       |          |   \fIf\fP    | record field                 |
       |          |   \fIL\fP    | library                      |
       +----------+--------+------------------------------+
       |          |   \fII\fP    | statement                    |
       | Non-     |   \fIE\fP    | expression                   |
       | termainal|   \fIV\fP    | pattern                      |
       |          |   \fIK\fP    | literal constant             |
       +----------+--------+------------------------------+\fR
.fi

.SS SYNTAX OF MODULES
.P
A module is the highest level syntactic construct. It contains the definition 
of lower level constructs. A GRL file contains exactly one module definition. 
This module has the same name as the file containing it.
.P
A module \fBP\fP can import libraries. Each library can be:
.IP -
another module \fBP\s-2i\s+2\fP. Therefore, the definitions of the imported 
module are visible in the main module \fBP\fP. Note that circular 
definitions are not allowed.
.IP -
a LNT module \fBmodule\fP. The file has extension lnt. See section external 
code for more information.
.IP -
a C filemane \fBfilename\fP. The file has extension c. See section external 
code for more information.
.P
.nf
.cs I 23
\fIprgm_def\fP ::= "module" \fIP\fP [ "(" \fIL\fP ( "," \fIL\fP )* ")" ] "is"
                (  \fItype_def\fP
                |  \fIconstant_def\fP
                |  \fIblock_def\fP
                |  \fIenvironment_def\fP
                |  \fImedium_def\fP
                |  \fIsystem_def\fP )*
             "end" "module"
.cs I
.fi

.SS SYNTAX OF TYPES
.P
.nf
.cs I 23
.ne 4v
\fItype\fP ::= "bool"
      |  "nat" |  "nat16" |  "nat32"
      |  "int" |  "int16" |  "int32"
      |  "char" |  "string"
      |  \fIT\fP
.cs I
.fi
.P
.nf
.cs I 23
.ne 4v
\fItype_def\fP ::= "type" \fIT\fP "is"
                \fItype_expr\fP
             "end" "type"
.cs I
.fi
.P
.nf
.cs I 23
.ne 4v
\fItype_expr\fP ::= "array" "[" \fIm\fP "..." \fIn\fP "]" "of" \fItype\fP
           |  "range" \fIm\fP "..." \fIn\fP
           |  "record" \fIf\fP ":" \fItype\fP ( "," \fIf\fP ":" \fItype\fP )*
           |  "enum" \fIC\fP ( "," \fIC\fP )*
.cs I
.fi
.P
User-defined types are the following:
.IP - 
The array type, defined using the keyword \fBarray\fP, denotes a fixed-size 
set of elements indexed by natural numbers ranging from \fBm\fP to \fBn\fP.
Where \fBm\fP and \fBn\fP are literal naturals.
.IP - 
The range type, defined using the keyword \fBrange\fP, denotes a finite 
interval of numbers ranging from \fBm\fP to \fBn\fP.
Where \fBm\fP and \fBn\fP are literal naturals.
.IP - 
The record type, defined using the keyword \fBrecord\fP, denotes a fixed-size 
tuple of elements indexed by field names \fBf\s-20\s+2, ..., f\s-2n\s+2\fP.
.IP - 
The enumerated type, defined using the keyword \fBenum\fP, denotes a finite 
and ordered set of symbolic values 
(identifiers) \fBC\s-20\s+2, ..., C\s-2n\s+2\fP.
.IP - 
The external type, defined in an imported LNT library, denotes any LNT type. 
An external type can be used natively in GRL modules.
.P
\fBExample\fP
.P
.nf
.cs I 23
.ne 4v    
\fB
type T_Array is
   array [0 ... 3] of bool
end type
.br

type T_Range is
   range -1 ... 1
end type
.br

type T_Record is
   record f1 : nat, f2 : bool
end type
.br

type T_Enum is
   enum a, b, c, d, e, f
end type
\fP
.cs I
.fi

.SS SYNTAX OF LITERAL CONSTANTS
.P
Literal constants may be either integer numbers, boolean constants, string 
constants or values of enumerated types.
.P
.nf
.cs I 23
.ne 4v
\fIK\fP ::= \fIint\fP
   |  \fIbool\fP
   |  \fIstring\fP
   |  \fIC\fP
.cs I
.fi

.SS SYNTAX OF OPERATORS
.P
.nf
.cs I 23
.ne 4v
\fIunary_op\fP ::= "+" | "-" | "not" | "abs" 
           | "nat" | "nat16" | "nat32" 
           | "int" | "int16" | "int32"
.cs I
.fi
.P
.nf
.cs I 23
.ne 4v
\fIbinary_op\fP ::= "and" | "or" | "xor" | "implies" | "equ"
            | "+" | "-" | "%" | "^" | "*" | "/"
            | "!=" | "==" | "<" | ">" | "<=" | ">="
.cs I
.fi

.SS SYNTAX OF EXPRESSIONS
.P
.nf
.cs I 23
.ne 4v
\fIE\fP ::= \fIX\fP

   |  "(" \fIE\fP ")"

   |  \fIE\fP"."\fIf\fP

   |  \fIE\fP "[" \fIE\fP "]"

   |  \fIunary_op\fP \fIE\fP

   |  \fIE\fP \fIbinary_op\fP \fIE\fP

   |  \fIK\fP [ "of" \fItype\fP ]

   |  \fIF\fP "(" \fIE\fP ( "," \fIE\fP )* ")"
.cs I
.fi

.SS SYNTAX OF PATTERNS
.P
Patterns may be either a literal constant or a wildcard.
.P
.nf
.cs I 23
.ne 4v
\fIV\fP ::= \fIK\fP | "any"
.cs I
.fi

.SS SYNTAX OF PREDEFINED FUNCTIONS
.P
Predefined functions are unary operations, binary operations, type conversion 
functions, functions on arrays, functions on records, and constructor of 
external types.
.IP - 
Type conversion functions, denoted \fBT (E)\fP, convert an 
expression \fBE\fP from one numerical data type to another numerical data 
type \fBT\fP. Numerical data types 
are: \fBnat\fP, \fBnat16\fP, \fBnat32\fP, \fBint\fP, \fBint16\fP, \fBint32\fP, 
and all the range types. An exception is raised if the value of \fBE\fP does 
not belong to the type \fBT\fP.
.IP - 
Given an array type defined by \fBtype\ T\ is array\ [m\ ...\ n]\ of\ Tarr 
end\ type\fP, two predefined functions T\ :\ Tarr^(n-m+1)\ ->\ T 
and T\ :\ Tarr\ ->\ T are automatically generated (those two functions 
coincide into one single function if m = n). The 
call \fBT\ (E\s-2m\s+2,\ ...,\ E\s-2n\s+2)\fP builds 
an array \fBX\fP in which each element \fBX[i]\fP is set to the value of 
expression \fBE\s-2i\s+2\fP. The call \fBT\ (E\s-20\s+2)\fP builds an 
array \fBX\fP in which all elements \fBX[i]\fP are set to the value of 
expression \fBE\s-20\s+2\fP.
.IP - 
Given a record type defined by \fBtype\ T\ is 
record\ f\s-20\s+2\ :\ Trec\s-20\s+2,\ ...,\ f\s-2n\s+2\ :\ Trec\s-2n\s+2 
end\ type\fP, a predefined 
function T\ :\ Trec\s-20\s+2\ x\ ...\ x\ Trec\s-2n\s+2\ ->\ T is automatically 
generated. 
The call \fBT\ (E\s-20\s+2,\ ...,\ E\s-2n\s+2)\fP returns a record in which 
each field \fBf\s-2i\s+2\fP is set to the value of expression \fBE\s-2i\s+2\fP.

.SS SYNTAX OF STATEMENTS
.P
.nf
.cs I 23
.ne 4v
\fII\fP ::= "null"

    | \fIX\fP ":=" \fIE\fP

    | \fIX\fP "[" \fIE\fP "]" ":=" \fIE\fP

    | \fIX\fP"."\fIf\fP ":=" \fIE\fP

    | \fII\fP ";" \fII\fP

    | "if" \fIE\fP "then" \fII\fP
      ( "elsif" \fIE\fP "then" \fII\fP )*
      [ "else" \fII\fP ]
      "end" "if"

    | "while" \fIE\fP "loop"
         \fII\fP
      "end" "loop"

    | "for" \fII\fP "while" \fIE\fP "by" \fII\fP "loop"
         \fII\fP
      "end" "loop"

    | "case" \fIE\fP ( "," \fIE\fP )* "is"
         \fIV\fP ( "," \fIV\fP )* "->" \fII\fP
         ( "|" \fIV\fP ( "," \fIV\fP )* "->" \fII\fP )*
      "end" "case"

    | "select"
         \fII\fP ( "[]" \fII\fP )*
      "end" "select"

    | \fIX\fP ":=" "any" \fItype\fP [ "where" \fIE\fP ]

    | "when" ["?"]\fIX\fP "->" \fII\fP

    | "when" ["?"] "<" \fIX\fP ( "," \fIX\fP )* ">" "->" \fII\fP

    | "enable" \fIX\fP

    | \fIBi\fP [ "{" \fIarg_list\fP "}" ] "(" \fIarg_list\fP ")"
.cs I
.fi

.SS SYNTAX OF CONSTANTS
.P
The keyword \fBconstant\fP is used to define a variable whose value, once 
initialized, can not be changed. A constant is visible and can be called by all 
other entities defined in the module.
.P
.nf
.cs I 23
.ne 4v
\fIconstant_def\fP ::= "const" \fIvar_list\fP ":" \fItype\fP ":=" \fIE\fP
.cs I
.fi
.P
\fBExample\fP
.P
.nf
.cs I 23
.ne 4v    
\fB
const C_Max : nat16 := 4
\fP
.cs I
.fi

.SS SYNTAX OF VARIABLE DECLARATIONS
.P
.nf
.cs I 23
.ne 4v
\fIdecl_list\fP ::= \fIvar_decl\fP ( "," \fIvar_decl\fP )*
\fIvar_decl\fP ::= \fIvar_list\fP ":" \fItype\fP [ ":=" \fIE\fP ]
\fIdecl_list_non_init\fP ::= \fIvar_decl_non_init\fP
                       ( "," \fIvar_decl_non_init\fP )*
\fIvar_decl_non_init\fP ::= \fIvar_list\fP ":" \fItype\fP
\fIvar_list\fP ::= \fIX\fP ( "," \fIX\fP )*
.cs I
.fi
.P
Permanent variables, defined after the keywords \fBstatic var\fP, have a 
lifetime extending across the entire execution of the program, whereas 
variables defined after the keyword \fBvar\fP are temporary.
.P
.nf
.cs I 23
.ne 4v
\fIlocal_vars\fP ::= "static" "var" \fIdecl_list\fP
             | "var" \fIdecl_list\fP
.cs I
.fi

.SS SYNTAX OF FORMAL PARAMETERS
.P
Formal parameters are classified as follows.
.IP - 
Constant parameters are user-fixed parameters defined as a set of variable 
declarations inside \fB{\fP and \fB}\fP. No value should be assigned
to such parameters after their definition.

.nf
.cs I 23
.ne 4v
\fIconst_param\fP ::= \fIdecl_list\fP
.cs I
.fi
.IP - 
Input/Output parameters are defined as a set of variable declarations 
preceded by the keywords \fBin\fP or \fBout\fP.

.nf
.cs I 23
.ne 4v
\fIinout_param\fP ::= "in" \fIdecl_list\fP
              | "out" \fIdecl_list_non_init\fP

\fIinout_param_non_init\fP ::= "in" \fIdecl_list_non_init\fP
                       | "out" \fIdecl_list_non_init\fP
.cs I
.fi
.IP - 
Communication parameters are defined as a set of variable declarations 
preceded by the keywords \fBreceive\fP or \fBsend\fP.

.nf
.cs I 23
.ne 4v
\fIcom_param\fP ::= "receive" \fIdecl_list_non_init\fP
            | "send" \fIdecl_list_non_init\fP
.cs I
.fi
.IP - 
Activation parameters are defined as a variable preceded by the 
keyword \fBblock\fP.

.nf
.cs I 23
.ne 4v
\fIactive_param\fP ::= "block" \fIX\fP ( "," \fIX\fP )*
.cs I
.fi

.SS SYNTAX OF BLOCKS
.P
Blocks represent the synchronous part, which is inspired by synchronous 
dataflow languages based on the block-diagram model. Following the definition 
of synchronous programs, all subblocks called in a block are governed by the 
clock this block.
.br
Subblocks can be aliased to increase readability using keyword \fBalias\fP. 
Constant parameters, if any, are mandatory when aliasing a block.
.P
.nf
.cs I 23
.ne 4v
\fIblock_def\fP ::=
   "block" \fIB\fP [ "{" \fIconst_param\fP "}" ]
             [ "(" \fIinout_param\fP ( "," \fIinout_param\fP )* ")" ]
             [ "[" \fIcom_param\fP ( "," \fIcom_param\fP )* "]" ] "is"
         [ "alias" \fIblock_alloc\fP ( "," \fIblock_alloc\fP )* ]
         [ \fIlocal_vars\fP ( "," \fIlocal_vars\fP )* ]
         \fII\fP
   "end" "block"

 | "block" \fIB\fP [ "{" \fIconst_param\fP "}" ]
             "(" \fIinout_param\fP ( "," \fIinout_param\fP )* ")" "is"
      "!c" \fIstring\fP | "!lnt" \fIstring\fP
   "end" "block"
.cs I
.fi
.P
A block definition can be either user-defined or included from an external code 
written in C or LNT language. An external block should not have receive nor 
send parameters and its body consists of a "pragma" denoting the language from 
which the external block is imported, followed by the name of the function 
implementing the block. See section external code for more information.
.P
\fBExample\fP
.P
The first example is a basic block allocating no other block, the second one 
connects several block instances together.
.P
.nf
.cs I 23
.ne 4v    
\fB
block B {c : nat}
        (in i : bool, j : nat, in k : bool, out o : nat) is
   if (i or k)
   then
      o := j
   else
      o := c
   end if
end block
.br

block B (in i : bool, j : nat, in k : bool, out o : nat) is
   alias B1 as Bi1; Bi2,
         B2 {true} as Bi3,
         B3 {_, _} as Bi4
   var c1, c3 : bool,
       c2 : nat
   Bi1 (i, j, ?c1);
   Bi3 (k, ?c2);
   Bi2 (c1, c2, ?c3);
   Bi4 (c3, ?o)
end block
\fP
.cs I
.fi

.SS SYNTAX OF ENVIRONMENTS
.P
Environments represents the behaviour of the environment surrounding a network 
of blocks. They allow to constrain either inputs of separate blocks or the 
relative order and frequency of block executions within a network. They make 
possible the description of the common environment in the case of parallel 
systems distributed on a single platform as well as a set of separate 
environments in the case of geographically distributed systems.
.P
.nf
.cs I 23
.ne 4v
\fIenv_param\fP ::= \fIactive_param\fP | \fIinout_param_non_init\fP
.br
\fIenvironment_def\fP ::=
   "environment" \fIN\fP [ "{" \fIconst_param\fP "}" ]
                   "(" \fIenv_param\fP ( "," \fIenv_param\fP )* ")" "is"
      [ "alias" \fIblock_alloc\fP ( "," \fIblock_alloc\fP )* ]
      [ \fIlocal_vars\fP ( "," \fIlocal_vars\fP )* ]
      \fII\fP
   "end" "environment"
.cs I
.fi
.P
An environment \fBN\fP interacts with blocks by connecting its input (resp., 
output) parameters to output (resp., input) parameters of blocks. Its body 
should define (or not) the behaviour of the environment when an interaction 
via a formal parameter list occurs. A signal of the 
form \fBwhen\ ?<X\s-20\s+2,\ ...,\ X\s-2n\s+2>\ ->\ I\fR defines the behaviour 
of \fBN\fP that corresponds to the parameter 
list \fBin\ X\s-20\s+2\ :\ T\s-20\s+2,\ ...,\ X\s-2n\s+2\ :\ T\s-2n\s+2\fR 
whereas a signal of the form \fBwhen\ <Y\s-20\s+2,\ ...,\ Y\s-2n\s+2>\ ->\ I\fR 
defines the behaviour of \fBN\fP that corresponds to the parameter 
list \fBout\ Y\s-20\s+2\ :\ T\s-20\s+2,\ ...,\ Y\s-2m\s+2\ :\ T\s-2m\s+2\fR.
.P
\fBExample\fP
.P
The first environment computes one output using global constant \fBC_Max\fP 
and \fBC_Init\fP. The second one computes an output using local constants and 
the value given by the input. The third one is an activation environment 
constaining block instances \fBA\fP and \fBB\fP.
.P
.nf
.cs I 23
.ne 4v    
\fB
environment E {Step : nat16 := 1} (out No : nat16) is
   static var Last_No : nat16 := C_Init
   when No -> if ((Last_No + Step) <= C_Max)
              then
                 No := Last_No + Step
              else
                 No := C_Init
              end if;
              Last_No := No
end environment
.br

environment E {Min    : nat16 := 0,
               Max    : nat16 := 30,
               Offset : nat16 := 3}
              (in X : nat16, out Y : nat16) is
   static var Last_X : nat16 := 0
   select
      when ?X -> Last_X := X
   []
      when Y -> select
                   Y := Last_X
                []
                   if (Last_X <= (Max - Step))
                   then
                      No := Last_X + Step
                   else
                      No := Last_X
                   end if
                []
                   if (Last_X >= (Min + Step))
                   then
                      No := Last_X - Step
                   else
                      No := Last_X
                   end if
                end select;
   end select
end environment
.br

environment E (block A, block B) is
   static var Last_Act_A : bool := false
   if (Last_Act_A)
   then
      enable B
   else
      enable A
   end if;
   Last_Act_A := not (Last_Act_A)
end environment
\fP
.cs I
.fi

.SS SYNTAX OF MEDIUMS
.P
Mediums represents the behaviour of communication mediums and enables a clean 
description of asynchronous interactions within a network of blocks. They 
enable the explicit description of the communication protocol and the rigorous 
design of networks whatever their topologies (star, bus, ring, etc.) and their 
means of communication (point-to-point, multipoint, etc.) are.
.P
.nf
.cs I 23
.ne 4v
\fImedium_def\fP ::=
   "medium" \fIM\fP [ "{" \fIconst_param\fP "}" ]
               "[" \fIcom_param\fP ( "," \fIcom_param\fP )* "]" "is"
      [ "alias" \fIblock_alloc\fP ( "," \fIblock_alloc\fP )* ]
      [ \fIlocal_vars\fP ( "," \fIlocal_vars\fP )* ]
      \fII\fP
   "end" "medium"

 | "medium" \fIM\fP [ "{" \fIconst_param\fP "}" ]
              [ "[" \fIcom_param\fP ( "," \fIcom_param\fP )* "]" ] "is"
      ( "from" "?" "<" \fIvar_list\fP ">" "to" "<" \fIvar_list\fP ">" )+
   "end" "medium"
.cs I
.fi
.P
A medium interacts with blocks by connecting its receive (resp., send) 
parameters to send (resp., receive) parameters of blocks. Its behaviour is the 
same as environments.
.P
For short mediums, \fBfrom\ ?<X\s-20\s+2,\ ...,\ X\s-2n\s+2>\fR should 
correspond to the parameter 
list \fBin\ X\s-20\s+2\ :\ T\s-20\s+2,\ ...,\ X\s-2n\s+2\ :\ T\s-2n\s+2\fR 
whereas \fBto\ <Y\s-20\s+2,\ ...,\ Y\s-2n\s+2>\fR should corresponds to the 
parameter 
list \fBout\ Y\s-20\s+2\ :\ T\s-20\s+2,\ ...,\ Y\s-2n\s+2\ :\ T\s-2n\s+2\fR.
.P
\fBExample\fP
.P
The first medium is a single buffer. The second medium is the short syntax for 
this buffer.
.P
.nf
.cs I 23
.ne 4v    
\fB
medium M [receive a : bool, c : bool,
          send b : bool, d : bool] is
   static var p : bool := false
              q : bool := false
   select
      when ?<a, c> -> p := a;
                      q := c
   []
      when <b, d> -> b := p;
                     d := q
   end select
end medium
.br

medium M [receive a : bool, c : bool,
          send b : bool, d : bool] is
   from ?<a, c> to <b, d>
end medium
\fP
.cs I
.fi

.SS SYNTAX OF INVOCATION
.P
Actual parameters, formally defined below, denote parameters passed to an 
instance at invocation time. 
.IP - 
The corresponding formal parameter of an actual parameter denotes the formal 
parameter that has the same position (as the actual parameter) in the actor 
definition.
.IP - 
An actual parameter is at constant position (respectively at input position,
at output position, at receive position, and at send position) if its 
corresponding formal parameter is defined inside \fB{\fP and \fB}\fP 
(respectively after keywords \fBin\fP, \fBout\fP, \fBreceive\fP, 
and \fBsend\fP).
.P
There are two types of actual parameters, the first one are used in systems and 
the second ones in other entities.
.IP - 
In system, an actual parameter can have different forms according to its 
position. A question mark precedes both actual parameters at output and send 
positions. An underscore is used for unconnected parameters. An actual 
parameter of the form \fBany type\fP assigns the corresponding formal 
parameters an arbitrary value of type \fBtype\fP.
.P
.nf
.cs I 23
.ne 4v
\fIchan_arg_list\fP ::= \fIchan_arg\fP ( "," \fIchan_args\fP )*

\fIchan_arg_list\fP ::= \fIchan_args\fP ( "," \fIchan_args\fP )*

\fIchan_args\fP ::= "<" \fIchan_arg\fP ( "," \fIchan_arg\fP )* ">"

\fIchan_arg\fP ::= ["?"] \fIX\fP | ["?"]_ | "any" \fItype\fP
.cs I
.fi
.IP - 
In other entities, an actual parameter can have different forms according to 
its position. A question mark precedes actual parameters at output positions. An 
underscore is used for unconnected parameters. An actual parameter of the form 
\fBany type\fP assigns the corresponding formal parameters an arbitrary value 
of type \fBtype\fP.
.P
.nf
.cs I 23
.ne 4v
\fIarg_list\fP ::= \fIarg\fP ( "," \fIarg\fP )*

\fIarg\fP ::= "?" \fIX\fP | ["?"]_ | \fIE\fP | "any" \fItype\fP
.cs I
.fi
.P
Instances can be allocated before being invoked. Actual parameters at 
constant position should be fixed at allocation time as follows.
.P
.nf
.cs I 23
.ne 4v
\fIblock_alloc\fP ::= \fIB\fP [ "{" \fIarg_list\fP "}" ] "as" \fIBi\fP ( ";" \fIBi\fP )*
.cs I
.fi
.P
.nf
.cs I 23
.ne 4v
\fIalloc\fP ::= \fIblock_alloc\fP
        | \fIN\fP [ "{" \fIarg_list\fP "}" ] "as" \fINi\fP ( ";" \fINi\fP )*
        | \fIM\fP [ "{" \fIarg_list\fP "}" ] "as" \fIMi\fP ( ";" \fIMi\fP )*
.cs I
.fi
.P
Instances can be invoked as follows. If instance has been allocated constant 
arguments are not allowed, else actual parameters at constant position should 
be fixed at invocation time.
.P
.nf
.cs I 23
.ne 4v
\fIblock_inst\fP ::= \fIBi\fP [ "{" \fIarg_list\fP "}" ] "(" \fIchan_arg_list\fP ")" 
             | \fIBi\fP [ "{" \fIarg_list\fP "}" ]
                  [ "(" \fIchan_arg_list\fP ")" ]
                  [ "[" \fIchan_arg_list\fP "]" ]

\fIenvironment_inst\fP ::= \fINi\fP [ "{" \fIarg_list\fP "}" ]
                        "(" \fIchan_arg_list\fP ")"

\fImedium_inst\fP ::= \fIMi\fP [ "{" \fIarg_list\fP "}" ]
                   "[" \fIchan_arg_list\fP "]"
.cs I
.fi

.SS SYNTAX OF SYSTEMS
.P
A system specifies a network of synchronous blocks. Those blocks are 
constrained by a set of environments and interact asynchronously via a set of 
mediums.
.P
.nf
.cs I 23
.ne 4v         
\fIsystem_def\fP ::=
   "system" \fIS\fP [ "{" \fIconst_param\fP "}" ]
              [ "(" \fIdecl_list_non_init\fP ")" ] "is"
      [ "alias" \fIalloc\fP ( "," \fIalloc\fP )* ]
      [ "var" \fIdecl_list_non_init\fP ]
      "block" "list"
         \fIblock_inst\fP ( "," \fIblock_inst\fP )*
      [ "environment" "list"
         \fIenvironment_inst\fP ( "," \fIenvironment_inst\fP )* ]
      [ "medium" "list"
         \fImedium_inst\fP ( "," \fImedium_inst\fP )* ]
   "end" "system"
.cs I
.fi
.P
The first list \fIdecl_list_non_init\fP defines the parameters that are visible 
from the external world whereas the second list \fIdecl_list_non_init\fP, after 
keyword \fBvar\fP, defines the invisible parameters.
.P
Note that activated block must be aliased before activated.
.P
\fBExample\fP
.P
.nf
.cs I 23
.ne 4v    
\fB
system Main {cst : nat16}
            (a, a2, b : int32, c : bool, d : nat) is
   alias B1 {cst} as Bi1,
         B2 {cst} as Bi2,
         E1 {_, 4} as Ei1,
         E2 {true, false} as Ei2,
         M1 as Mi1
   var w : nat16, x : nat16, y : nat16, z : nat16
   block list
      Bi1 (<a, a2>, ?<b>) [<w>, ?<x>],
      Bi2 (c, ?d) [y, ?z]
   environment list
      Ei1 (?<a, a2>),
      Ei2 (?c)
   medium list
      Mi1 [<x>, z, ?<w>, ?y]
end system
.br

system Main {cst : nat16}
            (a, a2, b : int32, c : bool, d : nat) is
   var w : nat16, x : nat16, y : nat16, z : nat16
   block list
      B1 {cst} (<a, a2>, ?<b>) [<w>, ?<x>],
      B2 {cst} (c, ?d) [y, ?z]
   environment list
      E1 {_, 4} (?<a, a2>),
      E2 {true, false} (?c)
   medium list
      M1 [<x>, z, ?<w>, ?y]
end system
.br

system Main (a, b, c, d : bool) is
   alias B1 as Bi1, B2 as Bi2 
   block list
      Bi1 (a, ?b),
      Bi2 (c, ?d)
   environment list
      E (Bi1, Bi2)
end system
\fP
.cs I
.fi

.SH EXTERNAL CODE
.P
External code can be either LNT code or C code.

.SS LNT CODE
.P
External LNT code should be written into .lnt files. A external block should be 
represented by a function in LNT code, following some rules.
.IP -
The number of parameters of the LNT functions and the block are the same.
.IP -
Input and constant parameters are represented by the \fBin\fP LNT keyword.
.br
Output parameters are represented by the \fBout\fP LNT keyword.
.IP -
Each LNT parameter should be typed using the equivalent LNT type.
The following LNT 
types \fBBool, Nat8, Nat16, Nat, Int8, Int16, Int, char, string\fP are 
respectively equivalent to the following 
types \fBbool, nat, nat16, nat32, int, int16, int32, char, string\fP.
.br
Note that user types keep the same name in the LNT code. And user LNT types can 
used natively in GRL modules.
.P
\fBExample\fP
.P
The following code gives the block and its LNT implementation.
.P
.nf
.cs I 23
.ne 4v    
\fB
block EXT (in a : int16, in a2 : int16,
           out b : int16, out b2 : int16) is
   !lnt "EXT_LNT"
end block
.br

function EXT_LNT (in a : Int16, in a2 : Int16,
                  out b : Int16, out b2 : Int16) is
   b := a + a2;
   b2 := a - a2
end function
\fP
.cs I
.fi

.SS C CODE
.P
External C code should be written into .c files. A external block should be 
represented by a function in C code, following some rules.
.IP -
The C function should have no side effect.
.IP - 
The number of parameters of the C functions and the block are the same.
.IP -
Input and constant parameters are represented by C value parameters.
.br
Output parameters are represented by C pointer parameters.
.IP -
Each C parameter should be typed using the equivalent C type.
The following C 
types \fBGRL_BOOL, GRL_NAT, GRL_NAT16, GRL_NAT32, GRL_INT, GRL_INT16, GRL_INT, 
GRL_CHAR, GRL_STRING\fP are respectively equivalent to the following 
types \fBbool, nat, nat16, nat32, int, int16, int32, char, string\fP.
.br
Note that user types are prefixed with the string "GRL_" and uppercase. For 
example the C type \fBGRL_TYPE_A\fP is equivalent to the type \fBtype_a\fP.
.IP -
It is recommended to use conversion functions (e.g., \fBGRL_TYPE1_TO_TYPE2\fP) 
at the beginning and at the end of each C function to interface safely with the 
GRL code. Each function converts the given value to the required type and 
checks that there is no overflow.
.P
\fBExample\fP
.P
The following code gives the block and its C implementation.
.P
.nf
.cs I 23
.ne 4v    
\fB
block EXT (in a : int16, in a2 : int16,
           out b : int16, out b2 : int16) is
   !c "EXT_C"
end block
.br

void EXT_C (GRL_INT16 a, GRL_INT16 a2,
            GRL_INT16* b, GRL_INT16* b2)
{
   signed short arg_a = GRL_INT16_TO_SIGNED_SHORT (a);
   signed short arg_a2 = GRL_INT16_TO_SIGNED_SHORT (a2);

   signed int res1 = arg_a + arg_a2;
   signed int res2 = arg_a - arg_a2;

   *b = GRL_SIGNED_INT_TO_INT16 (res1);
   *b2 = GRL_SIGNED_INT_TO_INT16 (res2);
}
\fP
.cs I
.fi

.SH AUTHORS
.P
Eric Leo (version 2).
.br
Eric Leo (version 1).
.br
Eric Leo (initial draft, based on Fatma Jebali's definition of GRL).

.SH SEE ALSO

For a complete description of GRL, see the \fBgrl2lnt\fP reference manual.
Otherwise see also the following man pages:
.br
.BR grl.open (LOCAL),
.BR grl2lnt (LOCAL).

.P
Additional information is available from the CADP Web page located at
http://cadp.inria.fr

.SH BUGS

Please report any bug to cadp@inria.fr

